@@ -16,6 +16,12 @@
 #'
 #'  While this function is intended to be very flexible (hence the large number of arguments),
 #'  the defaults should work in most cases for most users. See the \code{Examples}.
+#'
+#'  \strong{NEW}: Our
+#'  \href{https://huggingface.co/spaces/ludvigolsen/plot_confusion_matrix}{
+#'  \strong{Plot Confusion Matrix} web application}
+#'  allows using this function without code. Select from multiple design templates
+#'  or make your own.
 #' @author Ludvig Renbo Olsen, \email{r-pkgs@@ludvigolsen.dk}
 #' @export
 #' @family plotting functions
@@ -96,12 +102,21 @@
 #'  Either \code{`counts`}, \code{`normalized`} or one of \code{`log counts`,
 #'  `log2 counts`, `log10 counts`, `arcsinh counts`}.
 #'
-#'  For `normalized`, the color limits become \code{0-100}, why the intensities
+#'  For `normalized`, the color limits become \code{0-100} (except when
+#'  \code{`intensity_lims`} are specified), why the intensities
 #'  can better be compared across plots.
 #'
 #'  For the `log*` and `arcsinh` versions, the log/arcsinh transformed counts are used.
+#'
 #'  \strong{Note}: In `log*` transformed counts, 0-counts are set to `0`, why they
 #'  won't be distinguishable from 1-counts.
+#' @param intensity_lims A specific range of values for the color intensity of
+#'  the tiles. Given as a numeric vector with \code{c(min, max)}.
+#'
+#'  This allows having the same intensity scale across plots for better comparison
+#'  of prediction sets.
+#' @param intensity_beyond_lims What to do with values beyond the
+#'  \code{`intensity_lims`}. One of \code{"truncate", "grey"}.
 #' @param arrow_size Size of arrow icons. (Numeric)
 #'
 #'  Is divided by \code{sqrt(nrow(conf_matrix))} and passed on
@@ -117,6 +132,10 @@
 #'  Try these palettes: \code{"Greens"}, \code{"Oranges"},
 #'  \code{"Greys"}, \code{"Purples"}, \code{"Reds"},
 #'  as well as the default \code{"Blues"}.
+#'
+#'  Alternatively, pass a named list with limits of a custom gradient as e.g.
+#'  \code{`list("low"="#B1F9E8", "high"="#239895")`}. These are passed to
+#'  \code{\link[ggplot2:scale_fill_gradient]{ggplot2::scale_fill_gradient}}.
 #' @param tile_border_color Color of the tile borders. Passed as \emph{\code{`colour`}} to
 #' \code{\link[ggplot2:geom_tile]{ggplot2::geom_tile}}.
 #' @param tile_border_size Size of the tile borders. Passed as \emph{\code{`size`}} to
@@ -223,13 +242,24 @@
 #' )
 #'
 #' # Change color palette to green
-#' # Change theme to \code{theme_light}.
+#' # Change theme to `theme_light`.
 #' plot_confusion_matrix(
 #'   evaluation[["Confusion Matrix"]][[1]],
 #'   palette = "Greens",
 #'   theme_fn = ggplot2::theme_light
 #' )
 #'
+#' # Change colors palette to custom gradient
+#' # with a different gradient for sum tiles
+#' plot_confusion_matrix(
+#'   evaluation[["Confusion Matrix"]][[1]],
+#'   palette = list("low" = "#B1F9E8", "high" = "#239895"),
+#'   sums_settings = sum_tile_settings(
+#'     palette = list("low" = "#e9e1fc", "high" = "#BE94E6")
+#'   ),
+#'   add_sums = TRUE
+#' )
+#'
 #' # The output is a ggplot2 object
 #' # that you can add layers to
 #' # Here we change the axis labels
@@ -270,6 +300,8 @@ plot_confusion_matrix <- function(conf_matrix,
                                   counts_on_top = FALSE,
                                   palette = "Blues",
                                   intensity_by = "counts",
+                                  intensity_lims = NULL,
+                                  intensity_beyond_lims = "truncate",
                                   theme_fn = ggplot2::theme_minimal,
                                   place_x_axis_above = TRUE,
                                   rotate_y_text = TRUE,
@@ -313,9 +345,26 @@ plot_confusion_matrix <- function(conf_matrix,
   checkmate::assert_string(x = sub_col, min.chars = 1, add = assert_collection, null.ok = TRUE)
   checkmate::assert_string(x = tile_border_linetype, na.ok = TRUE, add = assert_collection)
   checkmate::assert_string(x = tile_border_color, na.ok = TRUE, add = assert_collection)
-  checkmate::assert_string(x = palette, na.ok = TRUE, null.ok = TRUE, add = assert_collection)
-  checkmate::assert_string(x = intensity_by, add = assert_collection)
   checkmate::assert_character(x = class_order, null.ok = TRUE, any.missing = FALSE, add = assert_collection)
+  checkmate::assert_string(x = intensity_by, add = assert_collection)
+  checkmate::assert_string(x = intensity_beyond_lims, add = assert_collection)
+  checkmate::assert_choice(x = intensity_beyond_lims, choices = c("truncate", "grey"), add = assert_collection)
+  checkmate::assert_string(x = sums_settings[["intensity_by"]], null.ok = TRUE, add = assert_collection)
+  checkmate::assert_string(x = sums_settings[["intensity_beyond_lims"]], null.ok = TRUE, add = assert_collection)
+  checkmate::assert_choice(x = sums_settings[["intensity_beyond_lims"]], choices = c("truncate", "grey"), null.ok = TRUE, add = assert_collection)
+
+
+  checkmate::assert(
+    checkmate::check_string(x = palette, na.ok = TRUE, null.ok = TRUE),
+    checkmate::check_list(x = palette, types = "character", names = "unique")
+  )
+  if (is.list(palette)){
+    checkmate::assert_names(
+      x = names(palette),
+      must.include = c("high", "low"),
+      add = assert_collection
+    )
+  }
 
   # Flag
   checkmate::assert_flag(x = add_sums, add = assert_collection)
@@ -340,6 +389,16 @@ plot_confusion_matrix <- function(conf_matrix,
   checkmate::assert_number(x = arrow_size, lower = 0, add = assert_collection)
   checkmate::assert_number(x = arrow_nudge_from_text, lower = 0, add = assert_collection)
   checkmate::assert_count(x = digits, add = assert_collection)
+  checkmate::assert_numeric(
+    x = intensity_lims,
+    len = 2,
+    null.ok = TRUE,
+    any.missing = FALSE,
+    unique = TRUE,
+    sorted = TRUE,
+    finite = TRUE,
+    add = assert_collection
+  )
 
   # List
   checkmate::assert_list(x = font_counts, names = "named", add = assert_collection)
@@ -381,6 +440,13 @@ plot_confusion_matrix <- function(conf_matrix,
     subset.of = c("counts", "normalized", "log counts", "log2 counts", "log10 counts", "arcsinh counts"),
     add = assert_collection
   )
+  if (!is.null(sums_settings[["intensity_by"]])){
+    checkmate::assert_names(
+      x = sums_settings[["intensity_by"]],
+      subset.of = c("counts", "normalized", "log counts", "log2 counts", "log10 counts", "arcsinh counts"),
+      add = assert_collection
+    )
+  }
 
   if (!is.null(class_order)){
     classes_in_data <- unique(conf_matrix[[target_col]])
@@ -393,10 +459,27 @@ plot_confusion_matrix <- function(conf_matrix,
     class_order <- sort(unique(conf_matrix[[target_col]]))
   }
 
-  if (!is.null(sums_settings[["palette"]]) &&
-      palette == sums_settings[["palette"]]){
-    assert_collection$push(
-      "'palette' and 'sums_settings[['palette']]' cannot be the same palette.")
+  # Check `sum_settings` `palette`
+  if (!is.null(sums_settings[["palette"]])) {
+    checkmate::assert(
+      checkmate::check_string(x = sums_settings[["palette"]], na.ok = TRUE),
+      checkmate::check_list(
+        x = sums_settings[["palette"]],
+        types = "character",
+        names = "unique"
+      )
+    )
+    if (is.list(sums_settings[["palette"]])) {
+      checkmate::assert_names(
+        x = names(sums_settings[["palette"]]),
+        must.include = c("high", "low"),
+        add = assert_collection
+      )
+    }
+
+    if (palettes_are_equal(palette, sums_settings[["palette"]])) {
+      assert_collection$push("'palette' and 'sums_settings[['palette']]' cannot be the same palette.")
+    }
   }
 
   # Check that N are (>= 0)
@@ -493,14 +576,18 @@ plot_confusion_matrix <- function(conf_matrix,
   #### Update sum tile settings ####
 
   if (isTRUE(add_sums)) {
-    if (palette == "Greens" &&
+
+    if (!is.list(palette) &&
+        palette == "Greens" &&
         is.null(sums_settings[["palette"]])) {
       sums_settings[["palette"]] <- "Blues"
     }
 
     # Defaults are defined in update_sum_tile_settings()
-    sums_settings <-
-      update_sum_tile_settings(sums_settings, defaults = list())
+    sums_settings <- update_sum_tile_settings(
+      settings = sums_settings,
+      defaults = list()
+    )
 
   }
 
@@ -513,7 +600,7 @@ plot_confusion_matrix <- function(conf_matrix,
                       "right" = get_figure_path("caret_forward_sharp.svg"))
 
   # Scale arrow size
-  arrow_size <- arrow_size / sqrt(nrow(conf_matrix))
+  arrow_size <- arrow_size / sqrt(nrow(conf_matrix) + as.integer(isTRUE(add_sums)))
 
   #### Prepare dataset ####
 
@@ -541,7 +628,15 @@ plot_confusion_matrix <- function(conf_matrix,
   # We need to do this before adding sums
   intensity_measures <- get_intensity_range(
     data = cm,
-    intensity_by = intensity_by
+    intensity_by = intensity_by,
+    intensity_lims = intensity_lims
+  )
+
+  # Handle intensities outside of the allow range¨
+  cm$Intensity <- handle_beyond_intensity_limits(
+    intensities = cm$Intensity,
+    intensity_range = intensity_measures,
+    intensity_beyond_lims = intensity_beyond_lims
   )
 
   # Add icons depending on where the tile will be in the image
@@ -627,14 +722,28 @@ plot_confusion_matrix <- function(conf_matrix,
     sum_data[nrow(sum_data), "Normalized_text"] <- ifelse(
       isTRUE(counts_on_top), "", paste0(sum_data[nrow(sum_data), "N"]))
 
+    sums_intensity_by <- sums_settings[["intensity_by"]]
+    if (is.null(sums_intensity_by)) sums_intensity_by <- intensity_by
+
+    sums_intensity_beyond_lims <- sums_settings[["intensity_beyond_lims"]]
+    if (is.null(sums_intensity_beyond_lims)) sums_intensity_beyond_lims <- intensity_beyond_lims
+
     # Set color intensity metric
-    sum_data <- set_intensity(sum_data, intensity_by)
+    sum_data <- set_intensity(sum_data, sums_intensity_by)
 
     # Get min and max intensity scores and their range
     # We need to do this before adding sum_data
     sums_intensity_measures <- get_intensity_range(
       data = head(sum_data, nrow(sum_data) - 1),
-      intensity_by = intensity_by
+      intensity_by = sums_intensity_by,
+      intensity_lims = sums_settings[["intensity_lims"]]
+    )
+
+    # Handle intensities outside of the allow range
+    sum_data[1:(nrow(sum_data) - 1), "Intensity"] <- handle_beyond_intensity_limits(
+      intensities = sum_data[1:(nrow(sum_data) - 1), "Intensity"]$Intensity,
+      intensity_range=sums_intensity_measures,
+      intensity_beyond_lims=sums_intensity_beyond_lims
     )
 
     # Get color limits
@@ -713,7 +822,7 @@ plot_confusion_matrix <- function(conf_matrix,
     ggplot2::labs(
       x = "Target",
       y = "Prediction",
-      fill = ifelse(intensity_by == "counts", "N", "Normalized"),
+      fill = ifelse(grepl("counts", intensity_by), "N", "Normalized"),
       label = "N"
     ) +
     ggplot2::geom_tile(
@@ -723,13 +832,23 @@ plot_confusion_matrix <- function(conf_matrix,
       show.legend = FALSE
     ) +
     theme_fn() +
-    ggplot2::coord_equal() +
-    # Add fill colors that differ by N
-    ggplot2::scale_fill_distiller(
-      palette = palette,
-      direction = 1,
-      limits = color_limits
-    ) +
+    ggplot2::coord_equal()
+
+
+  # Add fill colors that differ by N
+  if (is.list(palette)) {
+    pl <- pl +
+      ggplot2::scale_fill_gradient(low = palette[["low"]],
+                                   high = palette[["high"]],
+                                   limits = color_limits)
+  } else {
+    pl <- pl +
+      ggplot2::scale_fill_distiller(palette = palette,
+                                    direction = 1,
+                                    limits = color_limits)
+  }
+
+  pl <- pl +
     # Remove the guide
     ggplot2::guides(fill = "none") +
     ggplot2::theme(
@@ -758,12 +877,20 @@ plot_confusion_matrix <- function(conf_matrix,
           linewidth = sums_settings[["tile_border_size"]],
           linetype = sums_settings[["tile_border_linetype"]],
           show.legend = FALSE
-        ) +
-        ggplot2::scale_fill_distiller(
-          palette = sums_settings[["palette"]],
-          direction = 1,
-          limits = sums_color_limits
         )
+
+      if (is.list(sums_settings[["palette"]])) {
+        pl <- pl +
+          ggplot2::scale_fill_gradient(low = sums_settings[["palette"]][["low"]],
+                                       high = sums_settings[["palette"]][["high"]],
+                                       limits = sums_color_limits)
+      } else {
+        pl <- pl +
+          ggplot2::scale_fill_distiller(palette = sums_settings[["palette"]],
+                                        direction = 1,
+                                        limits = sums_color_limits)
+      }
+
     } else {
       pl <- pl +
         ggplot2::geom_tile(
